@@ -29,11 +29,16 @@ use List::Util qw(first);
 use List::MoreUtils qw(any);
 use Parse::DebianChangelog;
 
-use Lintian::Check qw(check_spelling);
+use Lintian::Check qw(check_spelling spelling_tag_emitter);
 use Lintian::Relation::Version qw(versions_gt);
 use Lintian::Tags qw(tag);
 use Lintian::Util qw(file_is_encoded_in_non_utf8 strip);
 
+my $SPELLING_ERROR_IN_NEWS
+  = spelling_tag_emitter('spelling-error-in-news-debian');
+my $SPELLING_ERROR_CHANGELOG
+  = spelling_tag_emitter('spelling-error-in-changelog');
+
 sub run {
     my ($pkg, undef, $info, undef, $group) = @_;
     my $found_html = 0;
@@ -76,11 +81,11 @@ sub run {
                 tag 'changelog-file-not-compressed', $basename;
             } else {
                 my $max_compressed = 0;
-                my $file_info = $info->file_info($path);
+                my $file_info = $path->file_info;
                 if ($path->is_symlink) {
                     my $normalized = $path->link_normalized;
                     if (defined($normalized)) {
-                        $file_info = $info->file_info($normalized);
+                        $file_info = $path->file_info;
                     }
                 }
                 if (defined($file_info)) {
@@ -132,8 +137,8 @@ sub run {
                 tag 'debian-news-entry-has-strange-distribution',
                   $news->Distribution;
             }
-            check_spelling('spelling-error-in-news-debian',
-                $news->Changes,undef, $group->info->spelling_exceptions);
+            check_spelling($news->Changes, $group->info->spelling_exceptions,
+                $SPELLING_ERROR_IN_NEWS);
             if ($news->Changes =~ /^\s*\*\s/) {
                 tag 'debian-news-entry-uses-asterisk';
             }
@@ -314,16 +319,28 @@ sub run {
             $first_upstream =~ s/-[^-]+$//;
             my $second_upstream = $second_version;
             $second_upstream =~ s/-[^-]+$//;
-            if (    $first_upstream eq $second_upstream
-                and $entries[0]->Changes
-                =~ /^\s*\*\s+new\s+upstream\s+(?:\S+\s+)?release\b/im) {
-                tag 'possible-new-upstream-release-without-new-version';
+            my $first_debian = $first_version;
+            $first_debian =~ s/^[^-]+-//;
+            my $second_debian = $second_version;
+            $second_debian =~ s/^[^-]+-//;
+
+            if ($first_upstream eq $second_upstream) {
+                if ($entries[0]->Changes
+                    =~ /^\s*\*\s+new\s+upstream\s+(?:\S+\s+)?release\b/im) {
+                    tag 'possible-new-upstream-release-without-new-version';
+                }
+                if ($first_debian =~ /^\d+$/ and $second_debian =~ /^\d+$/) {
+                    unless ($first_debian == $second_debian + 1) {
+                        tag 'non-consecutive-debian-revision';
+                    }
+                }
             }
 
             my $first_dist = lc $entries[0]->Distribution;
             my $second_dist = lc $entries[1]->Distribution;
             if ($first_dist eq 'unstable' and $second_dist eq 'experimental') {
-                unless ($entries[0]->Changes =~ /\bto\s+(?:unstable|sid)\b/im){
+                unless ($entries[0]->Changes
+                    =~ /\bto\s+['"‘“]?(?:unstable|sid)['"’”]?\b/im) {
                     tag 'experimental-to-unstable-without-comment';
                 }
             }
@@ -348,7 +365,7 @@ sub run {
             tag 'possible-missing-colon-in-closes', $1 if $1;
         }
 
-        # check for bad intented distribution
+        # check for bad intended distribution
         if (
             $changes =~ /uploads? \s+ to \s+
                             (?'intended'testing|unstable|experimental|sid)/xi
@@ -358,12 +375,20 @@ sub run {
                 $intended = 'unstable';
             }
             my $uploaded = $entry->Distribution;
-            unless($uploaded eq $intended) {
-                tag 'bad-intended-distibution',
-                  "intended to $intended but uploaded to $uploaded";
+            unless ($uploaded eq 'UNRELEASED') {
+                unless($uploaded eq $intended) {
+                    tag 'bad-intended-distribution',
+                      "intended to $intended but uploaded to $uploaded";
+                }
             }
         }
 
+        my $changesempty = $changes;
+        $changesempty =~ s,\W,,gms;
+        if (length($changesempty)==0) {
+            tag 'changelog-empty-entry';
+        }
+
         my $closes = $entry->Closes;
         for my $bug (@$closes) {
             tag 'improbable-bug-number-in-closes', $bug if ($bug < 100);
@@ -434,8 +459,8 @@ sub run {
         # Strip out all lines that contain the word spelling to avoid false
         # positives on changelog entries for spelling fixes.
         $changes =~ s/^.*spelling.*\n//gm;
-        check_spelling('spelling-error-in-changelog', $changes, undef,
-            $group->info->spelling_exceptions);
+        check_spelling($changes, $group->info->spelling_exceptions,
+            $SPELLING_ERROR_CHANGELOG);
     }
 
     return;
