@@ -28,20 +28,17 @@ use strict;
 use warnings;
 use autodie;
 use v5.10;
-use Carp qw(croak confess);
-
-use Cwd();
-use File::Find;
 
 # Half of the size used in the "sliding window" for detecting bad
 # licenses like GFDL with invariant sections.
 # NB: Keep in sync cruft-gfdl-fp-sliding-win/pre_build.
-use constant BLOCKSIZE => 4096;
+# not less than 8192 for source missing
+use constant BLOCKSIZE => 16_384;
 
 use Lintian::Data;
 use Lintian::Relation ();
 use Lintian::Tags qw(tag);
-use Lintian::Util qw(fail is_ancestor_of normalize_pkg_path strip);
+use Lintian::Util qw(fail normalize_pkg_path strip);
 use Lintian::SlidingWindow;
 
 # All the packages that may provide config.{sub,guess} during the build, used
@@ -105,7 +102,7 @@ my $WARN_FILE_TYPE =  Lintian::Data->new(
         $transformlist = strip($transformlist);
 
         my $syntaxerror = 'Syntax error in cruft/warn-file-type';
-        my @transformpairs = ();
+        my @transformpairs;
         unless($transformlist eq '') {
             my @transforms = split(/\s*\&\&\s*/, $transformlist);
             if(scalar(@transforms) > 0) {
@@ -180,6 +177,7 @@ sub _get_license_check_file {
                   \&_check_gfdl_license_problem,
                 'rfc-whitelist-filename' =>\&_rfc_whitelist_filename,
                 'php-source-whitelist' => \&_php_source_whitelist,
+                #'print-group'          => sub { print($1)},
             );
             my @splitline = split(/\s*\~\~\s*/, $_[1], 5);
             my $syntaxerror = 'Syntax error in '.$filename;
@@ -222,15 +220,15 @@ sub _get_license_check_file {
     return $data;
 }
 
-# get usual non distribuable license
+# get usual non distributable license
 my $NON_DISTRIBUTABLE_LICENSES
   = _get_license_check_file('cruft/non-distributable-license');
 
 # get non free license
-# get usual non distribuable license
+# get usual non distributable license
 my $NON_FREE_LICENSES = _get_license_check_file('cruft/non-free-license');
 
-# get usual datas about admissible/not admissible GFDL invariant part of license
+# get usual data about admissible/not admissible GFDL invariant part of license
 my $GFDL_FRAGMENTS = Lintian::Data->new(
     'cruft/gfdl-license-fragments-checks',
     qr/\s*\~\~\s*/,
@@ -280,19 +278,19 @@ our %ERRORS = (
 # source-contains or diff-contains).  Note that only one of these regexes
 # should trigger for any single directory.
 my @directory_checks = (
-    [qr,^(.+/)?CVS$,        => 'cvs-control-dir'],
-    [qr,^(.+/)?\.svn$,      => 'svn-control-dir'],
-    [qr,^(.+/)?\.bzr$,      => 'bzr-control-dir'],
-    [qr,^(.+/)?\{arch\}$,   => 'arch-control-dir'],
-    [qr,^(.+/)?\.arch-ids$, => 'arch-control-dir'],
-    [qr!^(.+/)?,,.+$!       => 'arch-control-dir'],
-    [qr,^(.+/)?\.git$,      => 'git-control-dir'],
-    [qr,^(.+/)?\.hg$,       => 'hg-control-dir'],
-    [qr,^(.+/)?\.be$,       => 'bts-control-dir'],
-    [qr,^(.+/)?\.ditrack$,  => 'bts-control-dir'],
+    [qr,^(.+/)?CVS/?$,        => 'cvs-control-dir'],
+    [qr,^(.+/)?\.svn/?$,      => 'svn-control-dir'],
+    [qr,^(.+/)?\.bzr/?$,      => 'bzr-control-dir'],
+    [qr,^(.+/)?\{arch\}/?$,   => 'arch-control-dir'],
+    [qr,^(.+/)?\.arch-ids/?$, => 'arch-control-dir'],
+    [qr!^(.+/)?,,.+/?$!       => 'arch-control-dir'],
+    [qr,^(.+/)?\.git/?$,      => 'git-control-dir'],
+    [qr,^(.+/)?\.hg/?$,       => 'hg-control-dir'],
+    [qr,^(.+/)?\.be/?$,       => 'bts-control-dir'],
+    [qr,^(.+/)?\.ditrack/?$,  => 'bts-control-dir'],
 
     # Special case (can only be triggered for diffs)
-    [qr,^(.+/)?\.pc$, => 'quilt-control-dir'],
+    [qr,^(.+/)?\.pc/?$, => 'quilt-control-dir'],
 );
 
 # File checks.  These regexes match files that shouldn't be in the source
@@ -307,7 +305,7 @@ my @file_checks = (
     [qr,^(.+/)?\.arch-inventory$,       => 'arch-inventory-file'],
     [qr,^(.+/)?\.hgtags$,               => 'hg-tags-file'],
     [qr,^(.+/)?\.\#(.+?)\.\d+(\.\d+)*$, => 'cvs-conflict-copy'],
-    [qr,^(.+/)?(.+?)\.(r\d+)$,          => 'svn-conflict-file'],
+    [qr,^(.+/)?(.+?)\.(r[1-9]\d*)$,     => 'svn-conflict-file'],
     [qr,\.(orig|rej)$,                  => 'patch-failure-file', 1],
     [qr,((^|/)\.[^/]+\.swp|~)$,         => 'editor-backup-file', 1],
 );
@@ -319,19 +317,16 @@ our @EOL_TERMINATORS_FILES = qw(control changelog);
 sub run {
     my (undef, undef, $info, $proc) = @_;
     my $source_pkg = $proc->pkg_src;
-    my $droot = $info->debfiles;
+    my $d_files = $info->index_resolved_path('debian/files');
 
-    if (-e "$droot/files" and not -z "$droot/files") {
+    if ($d_files and $d_files->is_file and $d_files->size != 0) {
         tag 'debian-files-list-in-source';
     }
 
     # This doens't really belong here, but there isn't a better place at the
     # moment to put this check.
-    my $version = $info->field('version');
+    my $version = $info->field('version', '0-1');
 
-    # If the version field is missing, assume it to be a native,
-    # maintainer upload as it is probably the most likely case.
-    $version = '0-1' unless defined $version;
     if ($info->native) {
         if ($version =~ /-/ and $version !~ /-0\.[^-]+$/) {
             tag 'native-package-with-dash-version';
@@ -346,11 +341,6 @@ sub run {
     # or libtool.
     my $atdinbd= $info->relation('build-depends-all')->implies($AUTOTOOLS);
     my $ltinbd  = $info->relation('build-depends-all')->implies($LIBTOOL);
-
-    # Create a closure so that we can pass our lexical variables into
-    # the find wanted function.  We don't want to make them global
-    # because we'll then leak that data across packages in a large
-    # Lintian run.
     my %warned;
     my $format = $info->field('format');
 
@@ -360,26 +350,29 @@ sub run {
     #   that may not be present.
     $format = '3.0 (quilt)' unless defined $format;
     if ($format =~ /^\s*2\.0\s*\z/ or $format =~ /^\s*3\.0\s*\(quilt\)/) {
-        my $wanted= sub { check_debfiles($info, qr/\Q$droot\E/, \%warned) };
-        find($wanted, $droot);
+        check_debian_dir($info, \%warned);
     }elsif (not $info->native) {
         check_diffstat($info->diffstat, \%warned);
     }
     find_cruft($source_pkg, $info, \%warned, $atdinbd, $ltinbd);
 
     for my $file (@EOL_TERMINATORS_FILES) {
-        my $path = $info->debfiles($file);
-        next if not -f $path or not is_ancestor_of($droot, $path);
-        open(my $fd, '<', $path);
+        my $path = $info->index_resolved_path("debian/$file");
+        next if not $path or not $path->is_open_ok;
+        my $fd = $path->open;
         while (my $line = <$fd>) {
             if ($line =~ m{ \r \n \Z}xsm) {
-                tag 'control-file-with-CRLF-EOLs', "debian/$file";
+                tag 'control-file-with-CRLF-EOLs', $path;
                 last;
             }
         }
         close($fd);
     }
 
+    if (my $pycompat = $info->index_resolved_path('debian/pycompat')) {
+        tag 'debian-pycompat-is-obsolete' if $pycompat->is_file;
+    }
+
     # Report any error messages from tar while unpacking the source
     # package if it isn't just tar cruft.
     for my $file (keys %ERRORS) {
@@ -480,34 +473,50 @@ sub check_diffstat {
 # 3.0 (quilt) packages where there is no Debian diff and hence no diffstat
 # output.  Record any files we warn about in $warned so that we don't warn
 # again when checking the full unpacked source.
-sub check_debfiles {
-    my ($info, $droot, $warned) = @_;
-    (my $name = $File::Find::name) =~ s,^$droot/,,;
+sub check_debian_dir {
+    my ($info, $warned) = @_;
+    my $droot = $info->index_resolved_path('debian/');
+    return if not $droot;
 
-    # Check for unwanted directories and files.  This really duplicates the
-    # find_cruft function and we should find a way to combine them.
-    if (-d) {
-        for my $rule (@directory_checks) {
-            if ($name =~ /$rule->[0]/) {
-                tag "diff-contains-$rule->[1]", "debian/$name";
-                $warned->{"debian/$name"} = 1;
+    my @worklist = $droot->children;
+
+    # Check for unwanted directories and files.  This really
+    # duplicates the find_cruft function and we should find a way to
+    # combine them.
+
+  ENTRY:
+    while (my $entry = shift(@worklist)) {
+        my $name     = $entry->name;
+
+        if ($entry->is_dir) {
+            # Remove the trailing slash (historically we never
+            # included the slash for these tags and there is no
+            # particular reason to change that now).
+            $name     = substr($name,     0, -1);
+
+            for my $rule (@directory_checks) {
+                if ($name =~ /$rule->[0]/) {
+                    tag "diff-contains-$rule->[1]", $name;
+                    $warned->{$name} = 1;
+                }
+            }
+            push(@worklist, $entry->children);
+        } else {
+
+            for my $rule (@file_checks) {
+                if ($name =~ /$rule->[0]/) {
+                    tag "diff-contains-$rule->[1]", $entry;
+                    $warned->{$name} = 1;
+                }
+            }
+
+            # Additional special checks only for the diff, not the
+            # full source.
+            if ($name =~ m@^debian/(?:.+\.)?substvars$@o) {
+                tag 'diff-contains-substvars', $entry;
             }
         }
     }
-
-    -f or return;
-
-    for my $rule (@file_checks) {
-        if ($name =~ /$rule->[0]/) {
-            tag "diff-contains-$rule->[1]", "debian/$name";
-            $warned->{"debian/$name"} = 1;
-        }
-    }
-
-    # Additional special checks only for the diff, not the full source.
-    if ($name =~ m@^(?:.+\.)?substvars$@o) {
-        tag 'diff-contains-substvars', "debian/$name";
-    }
     return;
 }
 
@@ -531,7 +540,6 @@ sub find_cruft {
         my $name     = $entry->name;
         my $basename = $entry->basename;
         my $dirname = $entry->dirname;
-        my $path;
         my $file_info;
 
         if ($entry->is_dir) {
@@ -598,7 +606,7 @@ sub find_cruft {
                 'license-problem-md5sum-non-free-file');
         }
 
-        $file_info = $info->file_info($name);
+        $file_info = $entry->file_info;
 
         # warn by file type
         foreach my $tag_filetype ($WARN_FILE_TYPE->all) {
@@ -618,9 +626,8 @@ sub find_cruft {
 
         # waf is not allowed
         if ($basename =~ /\bwaf$/) {
-            my $path   = $info->unpacked($entry);
             my $marker = 0;
-            open(my $fd, '<', $path);
+            my $fd = $entry->open;
             while (my $line = <$fd>) {
                 next unless $line =~ m/^#/o;
                 if ($marker && $line =~ m/^#BZ[h0][0-9]/o) {
@@ -635,6 +642,18 @@ sub find_cruft {
             close($fd);
         }
 
+        # Lena Söderberg image
+        if ($basename =~ /\blenn?a\b/i) {
+            if(    $file_info =~ /\bimage\b/i
+                or $file_info =~ /^Matlab v\d+ mat/i
+                or $file_info =~ /\bbitmap\b/i
+                or $file_info =~ /^PDF Document\b/i
+                or $file_info =~ /^Postscript Document\b/i) {
+                tag 'license-problem-non-free-img-lenna', $name;
+            }
+
+        }
+
         # here we check old upstream specification
         # debian/upstream should be a directory
         if (   $name eq 'debian/upstream'
@@ -642,6 +661,25 @@ sub find_cruft {
             tag 'debian-upstream-obsolete-path', $name;
         }
 
+        # Find mentioning of usr/lib/perl5 inside the packaging
+        if ($name =~ m{^debian/}) {
+            my $short = $entry->basename();
+
+            # Skip symlinks and other nasty stuff as well as
+            # debian/changelog
+            if (   $short ne 'changelog'
+                && $entry->is_file
+                && $entry->is_open_ok) {
+                my $contents = $entry->file_contents;
+
+                # ignore comments
+                $contents =~ s/#.*$//m;
+                if (index($contents, 'usr/lib/perl5') >= 0) {
+                    tag 'mentions-deprecated-usr-lib-perl5-directory', $short;
+                }
+            }
+        }
+
         if (   $basename eq 'doxygen.png'
             or $basename eq 'doxygen.sty') {
             unless ($source_pkg eq 'doxygen') {
@@ -667,7 +705,7 @@ sub find_cruft {
             }
         }elsif ($basename =~ m{\A config.(?:guess|sub) \Z}xsm
             and not $atdinbd){
-            open(my $fd, '<', $info->unpacked($entry));
+            my $fd = $entry->open;
             while (<$fd>) {
                 last
                   if $.> 10;  # it's on the 6th line, but be a bit more lenient
@@ -690,7 +728,7 @@ sub find_cruft {
         }elsif ($basename eq 'ltconfig' and not $ltinbd) {
             tag 'ancient-libtool', $name;
         }elsif ($basename eq 'ltmain.sh', and not $ltinbd) {
-            open(my $fd, '<', $info->unpacked($entry));
+            my $fd = $entry->open;
             while (<$fd>) {
                 if (/^VERSION=[\"\']?(1\.(\d)\.(\d+)(?:-(\d))?)/) {
                     my ($version, $major, $minor, $debian)=($1, $2, $3, $4);
@@ -704,8 +742,7 @@ sub find_cruft {
             }
             close($fd);
         }
-        full_text_check($source_pkg, $entry, $info, $name, $basename, $dirname,
-            $info->unpacked($entry));
+        full_text_check($source_pkg, $entry, $info, $name, $basename,$dirname);
     }
     return;
 }
@@ -781,7 +818,7 @@ sub check_missing_source {
 # note that it does not replace licensecheck(1)
 # and is only used for autoreject by ftp-master
 sub full_text_check {
-    my ($source_pkg, $entry, $info, $name, $basename, $dirname, $path) = @_;
+    my ($source_pkg, $entry, $info, $name, $basename, $dirname) = @_;
 
     # license string in debian/changelog are probably just change
     # Ignore these strings in d/README.{Debian,source}.  If they
@@ -793,17 +830,15 @@ sub full_text_check {
         return;
     }
 
-    open(my $fd, '<:raw', $path);
-    # allow to check only text file
+    my $fd = $entry->open(':raw');
+    # check only text files
     unless (-T $fd) {
         close($fd);
         return;
     }
 
     # some js file comments are really really long
-    my $sfd
-      = Lintian::SlidingWindow->new($fd, \&lc_block,
-        _is_javascript_but_not_minified($name) ? 8092 : 4096);
+    my $sfd= Lintian::SlidingWindow->new($fd, \&lc_block, BLOCKSIZE);
     my %licenseproblemhash;
 
     # we try to read this file in block and use a sliding window
@@ -815,6 +850,9 @@ sub full_text_check {
         my ($cleanedblock, %matchedkeyword);
         my $blocknumber = $sfd->blocknumber();
 
+        # Check for non-distributable files - this
+        # applies even to non-free, as we still need
+        # permission to distribute those.
         if(
             _license_check(
                 $source_pkg, $name,
@@ -827,8 +865,8 @@ sub full_text_check {
             return;
         }
 
-        # some license issues do not apply to non-free
-        # because these file are distribuable
+        # Skip the rest of the license checks for non-free
+        # sections.
         if ($info->is_non_free) {
             next BLOCK;
         }
@@ -842,7 +880,7 @@ sub full_text_check {
         # check only in block 0
         if($blocknumber == 0) {
             _search_in_block0($entry, $info, $name, $basename, $dirname,
-                $path, $block);
+                $block);
         }
     }
     close($fd);
@@ -862,7 +900,7 @@ sub _is_javascript_but_not_minified {
 
 # search something in block $0
 sub _search_in_block0 {
-    my ($entry, $info, $name, $basename, $dirname, $path, $block) = @_;
+    my ($entry, $info, $name, $basename, $dirname, $block) = @_;
 
     if(_is_javascript_but_not_minified($name)) {
         # exception sphinx documentation
@@ -872,9 +910,15 @@ sub _search_in_block0 {
                 return;
             }
         }
+        # false positive in dx package at least
+        elsif($basename eq 'srchidx.js') {
+            if($block =~ m/\A\s*profiles \s* = \s* new \s* Array\s*\(/xms) {
+                return;
+            }
+        }
         # see #745152
         # Be robust check also .js
-        if($basename eq 'deployJava.js') {
+        elsif($basename eq 'deployJava.js') {
             if($block =~ m/(?:\A|\v)\s*var\s+deployJava\s*=\s*function/xmsi) {
                 check_missing_source($entry,$info,$name,$basename,$dirname,
                     [['(?i)\.js$','.txt'],['','']]);
@@ -882,15 +926,14 @@ sub _search_in_block0 {
             }
         }
         # now search hidden minified
-        _linelength_test($entry, $info, $name, $basename, $dirname,
-            $path, $block);
+        _linelength_test($entry, $info, $name, $basename, $dirname,$block);
     }
     return;
 }
 
 # try to detect non human source based on line length
 sub _linelength_test {
-    my ($entry, $info, $name, $basename, $dirname, $path, $block) = @_;
+    my ($entry, $info, $name, $basename, $dirname, $block) = @_;
     my $strip = $block;
     # from perl faq strip comments
     $strip
@@ -933,13 +976,20 @@ sub _check_gfdl_license_problem {
     my $rawgfdlsections  = $matchedhash{rawgfdlsections}  || '';
     my $rawcontextbefore = $matchedhash{rawcontextbefore} || '';
 
-    # strip puntuation
+    # strip punctuation
     my $gfdlsections  = _strip_punct($rawgfdlsections);
     my $contextbefore = _strip_punct($rawcontextbefore);
 
-    my $oldgfdlsections;
+    # remove line number at beginning of line
+    # see krusader/1:2.4.0~beta3-2/doc/en_US/advanced-functions.docbook/
+    $gfdlsections =~ s{[ ]\d+[ ]}{ }gxsmo;
+    $gfdlsections =~ s{^\d+[ ]}{ }xsmo;
+    $gfdlsections =~ s{[ ]\d+$}{ }xsmo;
+    $gfdlsections =~ s{[ ]+}{ }xsmo;
+
     # remove classical and without meaning part of
     # matched string
+    my $oldgfdlsections;
     do {
         $oldgfdlsections = $gfdlsections;
         $gfdlsections =~ s{ \A \(?[ ]? g?fdl [ ]?\)?[ ]? [,\.;]?[ ]?}{}xsmo;
@@ -959,6 +1009,7 @@ sub _check_gfdl_license_problem {
                        permission [ ] is [ ] granted [ ] to [ ] copy [ ]?[,\.;]?[ ]?
                        distribute [ ]?[,\.;]?[ ]? and[ ]?/?[ ]?or [ ] modify [ ]
                        this [ ] document [ ] under [ ] the [ ] terms [ ] of [ ] the\Z}{}xsmo;
+
     # Treat ambiguous empty text
     unless(
         defined(
@@ -1006,7 +1057,7 @@ sub _check_gfdl_license_problem {
                 my $applytag = $gfdl_data->{'tag'};
                 if(defined($applytag)) {
                     unless(defined($licenseproblemhash->{$applytag})) {
-                        # lie will allow to check more block
+                        # lie will allow checking more blocks
                         _tag_gfdl($applytag, $name, $gfdlsections);
                         $licenseproblemhash->{$applytag} = 1;
                         return 0;
@@ -1070,7 +1121,7 @@ sub _clean_block {
                  (?:@[[:alpha:]]*?\{)?\s*gnu\s*\}                   # Tex info cmd
              }{ gnu }gxms;
 
-    # pod2man formating
+    # pod2man formatting
     $text =~ s{ \\ \* \( [LR] \" }{\"}gxsm;
     $text =~ s{ \\ -}{-}gxsm;
 
@@ -1087,12 +1138,13 @@ sub _clean_block {
     # po comment may include html tag
     $text =~ s/\"\s?\v\#~\s?\"//gxms;
 
-    $text =~ s/\\url{[^}]*?}/ /gxms;          # (la)?tex url
-    $text =~ s/\\emph{/ /gxms;                 # (la)?tex emph
-    $text =~ s/\\href{[^}]*?}
-                     {([^}]*?)}/ $1 /gxms;    # (la)?tex href
+    $text =~ s/\\url[{][^}]*?[}]/ /gxms;      # (la)?tex url
+    $text =~ s/\\emph[{]/ /gxms;              # (la)?tex emph
+    $text =~ s/\\href[{][^}]*?[}]
+                     [{]([^}]*?)[}]/ $1 /gxms;# (la)?tex href
     $text =~ s/\\hyperlink
-                 {[^}]*?}{([^}]*?)}/ $1 /gxms;# (la)?tex hyperlink
+                 [{][^}]*?[}]
+                 [{]([^}]*?)[}]/ $1 /gxms;    # (la)?tex hyperlink
     $text =~ s,-\\/,-,gxms;                   # tex strange hyphen
     $text =~ s,\\char, ,gxms;                 # tex  char command
 
@@ -1105,10 +1157,10 @@ sub _clean_block {
     $text =~ s/\@c(?:omment)?\s+/ /gxms;      # Tex info comment
 
     # Tex info bold,italic, roman, fixed width
-    $text =~ s/\@[birt]{/ /gxms;
-    $text =~ s/\@sansserif{/ /gxms;           # Tex info sans serif
-    $text =~ s/\@slanted{/ /gxms;             # Tex info slanted
-    $text =~ s/\@var{/ /gxms;                 # Tex info emphasis
+    $text =~ s/\@[birt][{]/ /gxms;
+    $text =~ s/\@sansserif[{]/ /gxms;         # Tex info sans serif
+    $text =~ s/\@slanted[{]/ /gxms;             # Tex info slanted
+    $text =~ s/\@var[{]/ /gxms;                 # Tex info emphasis
 
     $text =~ s/\@(?:small)?example\s+/ /gxms; # Tex info example
     $text =~ s/\@end \h+
@@ -1143,8 +1195,8 @@ sub _clean_block {
     $text =~ s/,\s*\"/ /gxms;
     $text =~ s/\\n/ /gxms;                    # Verbatim \n in string array
 
-    $text =~ s/\\&/ /gxms;                    # pod2man formating
-    $text =~ s/\\s(?:0|-1)/ /gxms;            # pod2man formating
+    $text =~ s/\\&/ /gxms;                    # pod2man formatting
+    $text =~ s/\\s(?:0|-1)/ /gxms;            # pod2man formatting
 
     $text =~ s/(?:``|'')/ /gxms;              # quote like
 
@@ -1169,7 +1221,7 @@ sub _clean_block {
     return $text;
 }
 
-# do not use space arround punctuation
+# do not use space around punctuation
 sub _strip_punct() {
     my ($text) = @_;
     # replace final punctuation
