@@ -46,9 +46,7 @@ our $known_build_essential
   = Lintian::Data->new('fields/build-essential-packages');
 our $KNOWN_BINARY_FIELDS = Lintian::Data->new('fields/binary-fields');
 our $KNOWN_UDEB_FIELDS = Lintian::Data->new('fields/udeb-fields');
-our $KNOWN_DEPENDENCY_RESTRICTIONS
-  = Lintian::Data->new('fields/dependency-restrictions',
-    qr/\./, \&_load_dependency_restrictions);
+our $KNOWN_BUILD_PROFILES = Lintian::Data->new('fields/build-profiles');
 
 our %KNOWN_ARCHIVE_PARTS = map { $_ => 1 } ('non-free', 'contrib');
 
@@ -57,7 +55,9 @@ my $KNOWN_PRIOS = Lintian::Data->new('common/priorities', qr/\s*=\s*/o);
 our @supported_source_formats = (qr/1\.0/, qr/3\.0\s*\((quilt|native)\)/);
 
 # Still in the archive but shouldn't be the primary Emacs dependency.
-our %known_obsolete_emacs = map { $_ => 1 } ('emacs21', 'emacs22');
+our %known_obsolete_emacs = map { $_ => 1 }
+  map { $_, $_.'-el', $_.'-gtk', $_.'-nox', $_.'-lucid' }
+  qw(emacs21 emacs22 emacs23);
 
 our %known_libstdcs = map { $_ => 1 } (
     'libstdc++2.9-glibc2.1', 'libstdc++2.10',
@@ -210,7 +210,7 @@ my %VCS_VALID_URIS = (
     bzr     => qr;^(?:sftp|(?:bzr\+)?ssh)://;,
     cvs     => qr;^(?:-d\s*)?:(?:ext|pserver):;,
     hg      => qr;^ssh://;,
-    git     => qr;^(?:git\+)?ssh://;,
+    git     => qr;^(?:git\+)?ssh://|^[\w.]+@[a-zA-Z0-9.]+:[/a-zA-Z0-9.];,
     svn     => qr;^(?:svn\+)?ssh://;,
 );
 
@@ -234,8 +234,7 @@ sub run {
     #---- Format
 
     if ($type eq 'source') {
-        my $format = $info->field('format');
-        if (defined($format)) {
+        if (defined(my $format = $info->field('format'))) {
             my $supported = 0;
             foreach my $f (@supported_source_formats){
                 if($format =~ /^\s*$f\s*\z/){
@@ -306,7 +305,7 @@ sub run {
             }
             if (
                 $upstream =~ m/[^~a-z]
-                    (rc|alpha|beta|dev|pre(?:view|release)?)
+                    (rc|alpha|beta|pre(?:view|release)?)
                    ([^a-z].*|\Z)
                  /xsm
               ) {
@@ -314,7 +313,7 @@ sub run {
                 my $rc = $1;
                 my $rest = $2//'';
                 my $suggestion;
-                # Remove the rc-part and the preceeding symbol (if any).
+                # Remove the rc-part and the preceding symbol (if any).
                 $expected =~ s/[\.\+\-\:]?\Q$rc\E.*//;
                 $suggestion = "$expected~$rc$rest";
                 tag 'rc-version-greater-than-expected-version', $upstream, '>',
@@ -350,8 +349,7 @@ sub run {
 
     #---- Multi-Arch
 
-    if (defined $info->field('multi-arch')){
-        my $march = $info->field('multi-arch');
+    if (defined(my $march = $info->field('multi-arch'))){
         unfold('multi-arch', \$march);
         tag 'unknown-multi-arch-value', $pkg, $march
           unless $march =~ m/^(?:no|foreign|allowed|same)$/o;
@@ -420,9 +418,7 @@ sub run {
 
     #---- Subarchitecture (udeb)
 
-    if (defined $info->field('subarchitecture')) {
-        my $subarch = $info->field('subarchitecture');
-
+    if (defined(my $subarch = $info->field('subarchitecture'))) {
         unfold('subarchitecture', \$subarch);
     }
 
@@ -441,6 +437,11 @@ sub run {
             unfold($f, \$maintainer);
 
             if ($f eq 'uploaders') {
+                # check for empty field see  #783628
+                if($maintainer =~ m/,\s*,/) {
+                    tag 'uploader-name-missing','you have used a double comma';
+                    $maintainer =~ s/,\s*,/,/g;
+                }
                 my %duplicate_uploaders;
                 my @uploaders = map { split /\@\S+\K\s*,\s*/ }
                   split />\K\s*,\s*/, $maintainer;
@@ -471,15 +472,10 @@ sub run {
 
     #---- Source
 
-    if (not defined $info->field('source')) {
-        # Optional in binary packages, required in source packages, but we
-        # cannot check it as dpkg-source(1) refuses to unpack source packages
-        # without this field (and fields indirectly depends on unpacked).  So
-        # "pass"...
-        1;
-    } else {
-        my $source = $info->field('source');
-
+    # Optional in binary packages, required in source packages, but we
+    # cannot check it as dpkg-source(1) refuses to unpack source packages
+    # without this field (and fields indirectly depends on unpacked)...
+    if (defined(my $source = $info->field('source'))) {
         unfold('source', \$source);
 
         if ($type eq 'source') {
@@ -505,9 +501,7 @@ sub run {
 
     #---- Essential
 
-    if (defined $info->field('essential')) {
-        my $essential = $info->field('essential');
-
+    if (defined(my $essential = $info->field('essential'))) {
         unfold('essential', \$essential);
 
         tag 'essential-in-source-package' if ($type eq 'source');
@@ -599,8 +593,7 @@ sub run {
 
     #--- Homepage
 
-    if (defined $info->field('homepage')) {
-        my $homepage = $info->field('homepage');
+    if (defined(my $homepage = $info->field('homepage'))) {
         my $orig = $homepage;
 
         unfold('homepage', \$homepage);
@@ -641,9 +634,7 @@ sub run {
 
     #---- Installer-Menu-Item (udeb)
 
-    if (defined $info->field('installer-menu-item')) {
-        my $menu_item = $info->field('installer-menu-item');
-
+    if (defined(my $menu_item = $info->field('installer-menu-item'))) {
         unfold('installer-menu-item', \$menu_item);
 
         $menu_item =~ /^\d+$/ or tag 'bad-menu-item', $menu_item;
@@ -675,7 +666,7 @@ sub run {
     if (($type eq 'binary') || ($type eq 'udeb')) {
         my $javalib = 0;
         my $replaces = $info->relation('replaces');
-        my %nag_once = ();
+        my %nag_once;
         $javalib = 1 if($pkg =~ m/^lib.*-java$/o);
         for my $field (
             qw(depends pre-depends recommends suggests conflicts provides enhances replaces breaks)
@@ -792,7 +783,7 @@ sub run {
 
                     tag 'package-depends-on-an-x-font-package',
                       "$field: $part_d_orig"
-                      if ( $field =~ /^(pre-)?depends$/
+                      if ( $field =~ /^(?:pre-)?depends$/
                         && $d_pkg =~ /^xfont.*/
                         && $d_pkg ne 'xfonts-utils'
                         && $d_pkg ne 'xfonts-encodings');
@@ -838,6 +829,13 @@ sub run {
                       && perl_core_has_version($d_pkg, $d_version->[0],
                         $d_version->[1]);
 
+                    tag 'package-relation-with-perl-modules', "$field: $d_pkg"
+                      # matches "perl-modules" (<= 5.20) as well as
+                      # perl-modules-5.xx (>> 5.20)
+                      if $d_pkg =~ /^perl-modules/
+                      && $field ne 'replaces'
+                      && $proc->pkg_src ne 'perl';
+
                     tag 'depends-exclusively-on-makedev', $field,
                       if ( $field eq 'depends'
                         && $d_pkg eq 'makedev'
@@ -991,29 +989,12 @@ sub run {
                             $restrictions_used = 1;
                         }
 
-                        for my $restr (@{$d_restr}) {
-                            my $dotcount = $restr =~ tr/.//;
-                            if ($dotcount != 1) {
-                                #<<< no tidy, tag name too long
-                                tag 'invalid-restriction-term-in-source-relation',
-                                #>>>
-                                  "$restr [$field: $part_d_orig]";
-                                next;
-                            }
-                            $restr =~ s/^!//;
-                            my ($ns, $label) = split(/\./, $restr, 2);
-                            if ($KNOWN_DEPENDENCY_RESTRICTIONS->known($ns)) {
-                                #<<< no tidy, tag name too long
-                                tag 'invalid-restriction-label-in-source-relation',
-                                #>>>
-                                  "$label [$field: $part_d_orig]"
-                                  unless any { $_ eq $label }
-                                @{$KNOWN_DEPENDENCY_RESTRICTIONS->value($ns)};
-                            } else {
-                                #<<< no tidy, tag name too long
-                                tag 'invalid-restriction-namespace-in-source-relation',
-                                #>>>
-                                  "$ns [$field: $part_d_orig]";
+                        for my $restrlist (@{$d_restr}) {
+                            for my $prof (@{$restrlist}) {
+                                $prof =~ s/^!//;
+                                tag 'invalid-profile-name-in-source-relation',
+                                  "$prof [$field: $part_d_orig]"
+                                  unless $KNOWN_BUILD_PROFILES->known($prof);
                             }
                         }
 
@@ -1023,7 +1004,7 @@ sub run {
                               $d_pkg;
                         }
 
-                        if ($d_pkg =~ m/^java-compiler$/o){
+                        if ($d_pkg eq 'java-compiler'){
                             tag 'build-depends-on-an-obsolete-java-package',
                               $d_pkg;
                         }
@@ -1083,6 +1064,10 @@ sub run {
                         tag 'bad-relation', "$field: $part_d_orig"
                           if $rest;
 
+                        tag 'bad-version-in-relation', "$field: $part_d_orig"
+                          if ($d_version->[0]
+                            && !version_check($d_version->[1]));
+
                         # only trigger this for the the preferred alternative
                         tag 'versioned-dependency-satisfied-by-perl',
                           "$field: $part_d_orig"
@@ -1132,25 +1117,27 @@ sub run {
         }
 
         # if restrictions are found in the build-depends/conflicts, then
-        # package must build-depend on dpkg (>= 1.17.2)
+        # package must build-depend on dpkg (>= 1.17.14)
         if ($restrictions_used) {
             my $build_conflicts_all = $info->relation('build-conflicts-all');
-            tag 'restriction-list-without-versioned-dpkg-dev-dependency'
-              unless ($build_all->implies('dpkg-dev (>= 1.17.2)'));
-            tag 'restriction-list-with-versioned-dpkg-dev-conflict'
+            tag 'restriction-formula-without-versioned-dpkg-dev-dependency'
+              unless ($build_all->implies('dpkg-dev (>= 1.17.14)'));
+            tag 'restriction-formula-with-versioned-dpkg-dev-conflict'
               if (
-                $build_conflicts_all->implies_inverse('dpkg-dev (<< 1.17.2)'));
+                $build_conflicts_all->implies_inverse('dpkg-dev (<< 1.17.14)')
+              );
             # if the package uses debhelper then it must require and not
-            # conflict with version >= 9.20140227
+            # conflict with version >= 9.20141010
             if ($build_all->implies('debhelper')) {
-                tag 'restriction-list-with-debhelper-without-debhelper-version'
-                  unless ($build_all->implies('debhelper (>= 9.20140227)'));
+                tag
+'restriction-formula-with-debhelper-without-debhelper-version'
+                  unless ($build_all->implies('debhelper (>= 9.20141010)'));
                 #<<< no tidy, tag name too long
-                tag 'restriction-list-with-debhelper-with-conflicting-debhelper-version'
+                tag 'restriction-formula-with-debhelper-with-conflicting-version'
                 #>>>
                   if (
                     $build_conflicts_all->implies_inverse(
-                        'debhelper (<< 9.20140227)'));
+                        'debhelper (<< 9.20141010)'));
             }
         }
 
@@ -1185,13 +1172,28 @@ sub run {
         if ($arch_dep_packages == 0 and $build_all->implies($PYTHON_DEV)) {
             tag 'build-depends-on-python-dev-with-no-arch-any';
         }
+
+        # libmodule-build-perl
+        # matches() instead of implies() because of possible OR releation
+        if ($info->relation('build-depends-indep')
+            ->matches(qr/^libmodule-build-perl$/, VISIT_PRED_NAME)
+            && !$info->relation('build-depends')
+            ->matches(qr/^libmodule-build-perl$/, VISIT_PRED_NAME)) {
+            tag 'libmodule-build-perl-needs-to-be-in-build-depends';
+        }
+
+        # libmodule-build-tiny-perl
+        if ($info->relation('build-depends-indep')
+            ->implies('libmodule-build-tiny-perl')
+            && !$info->relation('build-depends')
+            ->implies('libmodule-build-tiny-perl')) {
+            tag 'libmodule-build-tiny-perl-needs-to-be-in-build-depends';
+        }
     }
 
     #----- Origin
 
-    if (defined $info->field('origin')) {
-        my $origin = $info->field('origin');
-
+    if (defined(my $origin = $info->field('origin'))) {
         unfold('origin', \$origin);
 
         tag 'redundant-origin-field' if lc($origin) eq 'debian';
@@ -1199,9 +1201,7 @@ sub run {
 
     #----- Bugs
 
-    if (defined $info->field('bugs')) {
-        my $bugs = $info->field('bugs');
-
+    if (defined(my $bugs = $info->field('bugs'))) {
         unfold('bugs', \$bugs);
 
         tag 'redundant-bugs-field'
@@ -1210,9 +1210,7 @@ sub run {
 
     #----- Python-Version
 
-    if (defined $info->field('python-version')) {
-        my $pyversion = $info->field('python-version');
-
+    if (defined(my $pyversion = $info->field('python-version'))) {
         unfold('python-version', \$pyversion);
 
         my @valid = (
@@ -1222,6 +1220,11 @@ sub run {
             ['all']);
 
         my @pyversion = split(/\s*,\s*/, $pyversion);
+
+        if ($pyversion =~ m/^current/) {
+            tag 'python-version-current-is-deprecated';
+        }
+
         if (@pyversion > 2) {
             if (any { !/^\d+\.\d+$/ } @pyversion) {
                 tag 'malformed-python-version', $pyversion;
@@ -1248,9 +1251,7 @@ sub run {
 
     #----- Dm-Upload-Allowed
 
-    if (defined $info->field('dm-upload-allowed')) {
-        my $dmupload = $info->field('dm-upload-allowed');
-
+    if (defined(my $dmupload = $info->field('dm-upload-allowed'))) {
         tag 'dm-upload-allowed-is-obsolete';
 
         unfold('dm-upload-allowed', \$dmupload);
@@ -1315,19 +1316,18 @@ sub run {
     return;
 }
 
-# splits "foo:bar (>= 1.2.3) [!i386 ia64] <!profile.stage1 !profile.nocheck>" into
-# ( "foo", "bar", [ ">=", "1.2.3" ], [ [ "i386", "ia64" ], 1 ], [ "!profile.stage1" "!profile.nocheck" ], "" )
-#                                                         ^^^                                            ^^
-#                     count of negated arches, if ! was given                                            ||
-#                                                           rest (should always be "" for valid dependencies)
+# splits "foo:bar (>= 1.2.3) [!i386 ia64] <stage1 !nocheck> <cross>" into
+# ( "foo", "bar", [ ">=", "1.2.3" ], [ [ "i386", "ia64" ], 1 ], [ [ "stage1", "!nocheck" ] , [ "cross" ] ], "" )
+#                                                         ^^^                                               ^^
+#                     count of negated arches, if ! was given                                               ||
+#                                                              rest (should always be "" for valid dependencies)
 sub _split_dep {
     my $dep = shift;
     my ($pkg, $dmarch, $version, $darch, $restr)
       = ('', '', ['',''], [[], 0], []);
 
-    my $pkgname = $1 if $dep =~ s/^\s*([^<\s\[\(]+)\s*//;
-    if (defined $pkgname) {
-        ($pkg, $dmarch) = split /:/, $pkgname, 2;
+    if ($dep =~ s/^\s*([^<\s\[\(]+)\s*//) {
+        ($pkg, $dmarch) = split(/:/, $1, 2);
         $dmarch //= '';  # Ensure it is defined (in case there is no ":")
     }
 
@@ -1345,25 +1345,15 @@ sub _split_dep {
             }
             $darch->[1] = $negated;
         }
-        if ($dep && $dep =~ s/\s*<([^>]+)>\s*//) {
+        while ($dep && $dep =~ s/\s*<([^>]+)>\s*//) {
             my $t = $1;
-            $restr = [split /\s+/, $t];
+            push @$restr, [split /\s+/, $t];
         }
     }
 
     return ($pkg, $dmarch, $version, $darch, $restr, $dep);
 }
 
-sub _load_dependency_restrictions {
-    my ($key, $value, $pval) = @_;
-    my $ret = undef;
-    if (not defined $pval) {
-        $ret = $pval = [];
-    }
-    push @{$pval}, $value;
-    return $ret;
-}
-
 sub perl_core_has_version {
     my ($package, $op, $version) = @_;
     my $core_version = $PERL_CORE_PROVIDES->value($package);
