@@ -29,18 +29,18 @@ use File::Basename;
 use List::MoreUtils qw(any none);
 use Text::ParseWords ();
 
-use Lintian::Check qw(check_spelling);
+use Lintian::Check qw(check_spelling spelling_tag_emitter);
 use Lintian::Tags qw(tag);
 use Lintian::Util qw(clean_env drain_pipe fail open_gz);
 
 sub run {
     my ($pkg, undef, $info, $proc, $group) = @_;
     my $ginfo = $group->info;
-    my (%binary, %link, %manpage);
+    my (%binary, %link, %manpage, @running_man, @running_lexgrog);
 
     # Read package contents...
     foreach my $file ($info->sorted_index) {
-        my $file_info = $info->file_info($file);
+        my $file_info = $file->file_info;
         my $link = $file->link || '';
         my ($fname, $path, $suffix) = fileparse($file);
 
@@ -168,12 +168,12 @@ sub run {
                 }
             }
         } else { # not a symlink
-            my $fs_path = $info->unpacked($file);
+            my $fs_path = $file->fs_path;
             my $fd;
             if ($file_info =~ m/gzip compressed/) {
-                $fd = open_gz($fs_path);
+                $fd = $file->open_gz;
             } else {
-                open($fd, '<', $fs_path);
+                $fd = $file->open;
             }
             my @manfile = <$fd>;
             close $fd;
@@ -228,21 +228,12 @@ sub run {
                     exec('lexgrog', $fs_path)
                       or fail "exec lexgrog failed: $!";
                 }
-                my $desc = <$lexgrog_fd>;
-                $desc =~ s/^[^:]+: \"(.*)\"$/$1/;
-                if ($desc =~ /(\S+)\s+-\s+manual page for \1/i) {
-                    tag 'manpage-has-useless-whatis-entry', $file;
-                } elsif ($desc =~ /\S+\s+-\s+programs? to do something/i) {
-                    tag 'manpage-is-dh_make-template', $file;
-                }
-                drain_pipe($lexgrog_fd);
-                eval {close($lexgrog_fd);};
-                if (my $err = $@) {
-                    # Problem closing the pipe?
-                    fail "close pipe: $err" if $err->errno;
-                    # No, then lexgrog returned with a non-zero exit code.
-                    tag 'manpage-has-bad-whatis-entry', $file;
+                if (@running_lexgrog > 2) {
+                    process_lexgrog_output(\@running_lexgrog);
                 }
+                # lexgrog can have a high start up time, so revisit
+                # this later.
+                push(@running_lexgrog, [$file, $lexgrog_fd, $pid]);
             }
 
             # If it's not a .so link, run it through 'man' to check for errors.
@@ -278,45 +269,18 @@ sub run {
                 # parent - close write end
                 close $write;
             }
-            while (<$read>) {
-                # Devel::Cover causes some annoying deep recursion
-                # warnings and sometimes in our child process.
-                # Filter them out, but only during coverage.
-                next if LINTIAN_COVERAGE and m{
-                    \A Deep [ ] recursion [ ] on [ ] subroutine [ ]
-                    "[^"]+" [ ] at [ ] .*B/Deparse.pm [ ] line [ ]
-                   \d+}xsm;
-                # ignore progress information from man
-                next if /^Reformatting/;
-                next if /^\s*$/;
-                # ignore errors from gzip, will be dealt with at other places
-                next if /^(?:man|gzip)/;
-                # ignore wrapping failures for Asian man pages (groff problem)
-                if ($lang =~ /^(?:ja|ko|zh)/) {
-                    next if /warning \[.*\]: cannot adjust line/;
-                    next if /warning \[.*\]: can\'t break line/;
-                }
-                # ignore wrapping failures if they contain URLs (.UE is an
-                # extension for marking the end of a URL).
-                next
-                  if
-                  /:(\d+): warning \[.*\]: (?:can\'t break|cannot adjust) line/
-                  and ($manfile[$1 - 1] =~ m,(?:https?|ftp|file)://.+,i
-                    or $manfile[$1 - 1] =~ m,^\s*\.\s*UE\b,);
-                # ignore common undefined macros from pod2man << Perl 5.10
-                next if /warning: (?:macro )?\`(?:Tr|IX)\' not defined/;
-                chomp;
-                s/^[^:]+://o;
-                tag 'manpage-has-errors-from-man', $file, $_;
-                last;
+
+            if (@running_man > 3) {
+                process_man_output(\@running_man);
             }
-            close $read;
-            # reap man process
-            waitpid $pid, 0;
+            # man can have a high start up time, so revisit this
+            # later.
+            push(@running_man, [$file, $read, $pid, $lang, \@manfile]);
+
             # Now we search through the whole man page for some common errors
             my $lc = 0;
-            my $hc = 0;
-            my $draft_mode = 0;
+            my $stag_emitter
+              = spelling_tag_emitter('spelling-error-in-manpage', $file);
             foreach my $line (@manfile) {
                 $lc++;
                 chomp $line;
@@ -329,28 +293,6 @@ sub run {
                           "$file:$lc $fn_section != $th_section";
                     }
                 }
-                # Catch hyphens used as minus signs by looking for
-                # ones at the beginning of a word, but don't generate
-                # false positives on \s-1 (small font), \*(-- (pod2man
-                # long dash), or things like \h'-1'.  Ignoring hyphens
-                # contained in draft mode (.eo).  Do catch hyphens
-                # after \f[C], a groff font change.
-                $draft_mode = 1 if $line =~ /^\.\s*eo/;
-                $draft_mode = 0 if $line =~ /^\.\s*ec/;
-                if (
-                    not $draft_mode
-                    and $line =~ /^(
-                                ([^\.].*)?
-                                [\s\'\"\`\(\[\]]
-                                (?<! \\s | \*\( | \(- | \w\' )
-                               )?
-                              (--?\w+)/ox
-                  ) {
-                    $hc++;
-                    tag 'hyphen-used-as-minus-sign', "$file:$lc"
-                      if $hc <= 10
-                      or $ENV{'LINTIAN_DEBUG'};
-                }
                 if (($line =~ m,(/usr/(dict|doc|etc|info|man|adm|preserve)/),o)
                     || ($line =~ m,(/var/(adm|catman|named|nis|preserve)/),o)){
                     # FSSTND dirs in man pages
@@ -361,15 +303,15 @@ sub run {
                     tag 'manpage-has-errors-from-pod2man', "$file:$lc";
                 }
                 # Check for spelling errors if the manpage is English
-                check_spelling('spelling-error-in-manpage', $line, $file,
-                    $ginfo->spelling_exceptions)
+                check_spelling($line, $ginfo->spelling_exceptions,
+                    $stag_emitter)
                   if ($path =~ m,/man/man\d/,);
             }
-            tag 'hyphen-used-as-minus-sign', $file, ($hc - 10),
-              'more occurrences not shown'
-              if ($hc > 10 && !$ENV{'LINTIAN_DEBUG'});
         }
     }
+    # If we have any running sub processes, wait for them here.
+    process_lexgrog_output(\@running_lexgrog) if @running_lexgrog;
+    process_man_output(\@running_man) if @running_man;
 
     # Check our dependencies:
     foreach my $depproc (@{ $ginfo->direct_dependencies($proc) }) {
@@ -404,6 +346,73 @@ sub run {
     return;
 }
 
+sub process_lexgrog_output {
+    my ($running) = @_;
+    for my $lex_proc (@{$running}) {
+        my ($file, $lexgrog_fd, $pid) = @{$lex_proc};
+        my $desc = <$lexgrog_fd>;
+        $desc =~ s/^[^:]+: \"(.*)\"$/$1/;
+        if ($desc =~ /(\S+)\s+-\s+manual page for \1/i) {
+            tag 'manpage-has-useless-whatis-entry', $file;
+        } elsif ($desc =~ /\S+\s+-\s+programs? to do something/i) {
+            tag 'manpage-is-dh_make-template', $file;
+        }
+        drain_pipe($lexgrog_fd);
+        eval {close($lexgrog_fd);};
+        if (my $err = $@) {
+            # Problem closing the pipe?
+            fail "close pipe: $err" if $err->errno;
+            # No, then lexgrog returned with a non-zero exit code.
+            tag 'manpage-has-bad-whatis-entry', $file;
+        }
+    }
+    @{$running} = ();
+    return;
+}
+
+sub process_man_output {
+    my ($running) = @_;
+    for my $man_proc (@{$running}) {
+        my ($file, $read, $pid, $lang, $contents) = @{$man_proc};
+        while (<$read>) {
+            # Devel::Cover causes some annoying deep recursion
+            # warnings and sometimes in our child process.
+            # Filter them out, but only during coverage.
+            next if LINTIAN_COVERAGE and m{
+                    \A Deep [ ] recursion [ ] on [ ] subroutine [ ]
+                    "[^"]+" [ ] at [ ] .*B/Deparse.pm [ ] line [ ]
+                   \d+}xsm;
+            # ignore progress information from man
+            next if /^Reformatting/;
+            next if /^\s*$/;
+            # ignore errors from gzip, will be dealt with at other places
+            next if /^(?:man|gzip)/;
+            # ignore wrapping failures for Asian man pages (groff problem)
+            if ($lang =~ /^(?:ja|ko|zh)/) {
+                next if /warning \[.*\]: cannot adjust line/;
+                next if /warning \[.*\]: can\'t break line/;
+            }
+            # ignore wrapping failures if they contain URLs (.UE is an
+            # extension for marking the end of a URL).
+            next
+              if/:(\d+): warning \[.*\]: (?:can\'t break|cannot adjust) line/
+              and ($contents->[$1 - 1] =~ m,(?:https?|ftp|file)://.+,i
+                or $contents->[$1 - 1] =~ m,^\s*\.\s*UE\b,);
+            # ignore common undefined macros from pod2man << Perl 5.10
+            next if /warning: (?:macro )?\`(?:Tr|IX)\' not defined/;
+            chomp;
+            s/^[^:]+://o;
+            tag 'manpage-has-errors-from-man', $file, $_;
+            last;
+        }
+        close($read);
+        # reap man process
+        waitpid($pid, 0);
+    }
+    @{$running} = ();
+    return;
+}
+
 1;
 
 # Local Variables:
