@@ -31,7 +31,7 @@ use Dpkg::Vendor qw(get_current_vendor get_vendor_info);
 
 use Lintian::CheckScript;
 use Lintian::Tags;
-use Lintian::Util qw(parse_boolean read_dpkg_control strip);
+use Lintian::Util qw(parse_boolean read_dpkg_control_utf8 strip);
 
 =head1 NAME
 
@@ -352,7 +352,7 @@ sub _read_profile {
     my $pmap = $self->{'parent-map'};
     my $pname;
     my $plist = $self->{'profile_list'};
-    @pdata = read_dpkg_control($pfile, 0);
+    @pdata = read_dpkg_control_utf8($pfile, 0);
     $pheader = shift @pdata;
     croak "Profile field is missing from $pfile"
       unless defined $pheader && $pheader->{'profile'};
@@ -578,7 +578,7 @@ sub _check_for_invalid_fields {
 
 sub _load_check {
     my ($self, $profile, $check) = @_;
-    my $dir = undef;
+    my $dir;
     foreach my $checkdir ($self->_safe_include_path('checks')) {
         my $cf = "$checkdir/${check}.desc";
         if (-f $cf) {
@@ -636,7 +636,7 @@ sub _load_checks {
 }
 
 sub _default_inc_path {
-    my @path = ();
+    my @path;
     push @path, "$ENV{'HOME'}/.lintian"
       if exists $ENV{'HOME'} and defined $ENV{'HOME'};
     push @path, '/etc/lintian';
