@@ -54,6 +54,8 @@ BEGIN {
           visit_dpkg_paragraph
           parse_dpkg_control
           read_dpkg_control
+          read_dpkg_control_utf8
+          dpkg_deb_has_ctrl_tarfile
           get_deb_info
           get_dsc_info
           get_file_checksum
@@ -81,8 +83,11 @@ BEGIN {
           signal_number2name
           dequote_name
           load_state_cache
+          save_state_cache
           find_backlog
           unix_locale_split
+          pipe_tee
+          untaint
           $PKGNAME_REGEX),
         @{ $EXPORT_TAGS{constants} });
 }
@@ -90,10 +95,11 @@ BEGIN {
 use Digest::MD5;
 use Digest::SHA;
 use Encode ();
+use File::Temp qw(tempfile);
 use FileHandle;
 use Scalar::Util qw(openhandle);
 
-use Lintian::Command qw(spawn);
+use Lintian::Command qw(spawn safe_qx);
 use Lintian::Relation::Version qw(versions_equal versions_comparator);
 
 =head1 NAME
@@ -115,7 +121,7 @@ Lintian::Util - Lintian utility functions
  }
  
  my (@paragraphs);
- eval { @paragraphs = read_dpkg_control('some/debian/ctrl/file'); };
+ eval { @paragraphs = read_dpkg_control_utf8('some/debian/ctrl/file'); };
  if ($@) {
     # syntax error etc.
     die "ctrl/file: $@";
@@ -159,14 +165,14 @@ You have a I<.dsc> (or I<.changes>) file.  Alternative, it is also
 useful if you have a control file and only care about the first
 paragraph.
 
-=item Use L</read_dpkg_control> when
+=item Use L</read_dpkg_control_utf8> or L</read_dpkg_control> when
 
 You have a debian control file (such I<debian/control>) and you want
 a number of paragraphs from it.
 
 =item Use L</parse_dpkg_control> when
 
-When you would have used L</read_dpkg_control>, except you have an
+When you would have used L</read_dpkg_control_utf8>, except you have an
 open filehandle rather than a file name.
 
 =back
@@ -411,7 +417,7 @@ sub visit_dpkg_paragraph {
         # According to http://tools.ietf.org/html/rfc4880#section-6.2
         # The header MUST start at the beginning of the line and MUST NOT have
         # any other text (except whitespace) after the header.
-        elsif (m/^-----BEGIN PGP SIGNATURE-----\s*$/)
+        elsif (m/^-----BEGIN PGP SIGNATURE-----[ \r\t]*$/)
         { # skip until end of signature
             my $saw_end = 0;
             if (not $signed or $signature) {
@@ -425,7 +431,7 @@ sub visit_dpkg_paragraph {
             }
             $signature = $.;
             while (<$CONTROL>) {
-                if (m/^-----END PGP SIGNATURE-----\s*$/o) {
+                if (m/^-----END PGP SIGNATURE-----[ \r\t]*$/o) {
                     $saw_end = 1;
                     last;
                 }
@@ -449,7 +455,7 @@ sub visit_dpkg_paragraph {
             #    - Valid, but we don't support partial messages, so
             #      bail on those.
 
-            unless (m/^-----BEGIN PGP SIGNED MESSAGE-----\s*$/) {
+            unless (m/^-----BEGIN PGP SIGNED MESSAGE-----[ \r\t]*$/) {
                 # Not a (full) PGP MESSAGE; reject.
 
                 my $key = qr/(?:BEGIN|END) PGP (?:PUBLIC|PRIVATE) KEY BLOCK/;
@@ -457,7 +463,7 @@ sub visit_dpkg_paragraph {
                 my $msg
                   = qr/(?:BEGIN|END) PGP (?:(?:COMPRESSED|ENCRYPTED) )?MESSAGE/;
 
-                if (m/^-----($key|$msgpart|$msg)-----\s*$/o) {
+                if (m/^-----($key|$msgpart|$msg)-----[ \r\t]*$/o) {
                     die "syntax error at line $.: Unexpected $1 header\n";
                 } else {
                     die "syntax error at line $.: Malformed PGP header\n";
@@ -474,7 +480,7 @@ sub visit_dpkg_paragraph {
                     # allow two paragraphs to merge.  Consider:
                     #
                     # Field-P1: some-value
-                    # -----BEGIN PGP SIGANTURE----
+                    # -----BEGIN PGP SIGNATURE-----
                     #
                     # Field-P2: another value
                     #
@@ -504,9 +510,9 @@ sub visit_dpkg_paragraph {
             # two paragraphs to merge.  Consider:
             #
             # Field-P1: some-value
-            # -----BEGIN PGP SIGANTURE----
+            # -----BEGIN PGP SIGNATURE-----
             # [...]
-            # -----END PGP SIGANTURE----
+            # -----END PGP SIGNATURE-----
             # Field-P2: another value
             #
             # At the time of writing: If $open_section is true, it
@@ -589,6 +595,8 @@ sub visit_dpkg_paragraph {
     }
 }
 
+=item read_dpkg_control_utf8(FILE[, FLAGS[, LINES]])
+
 =item read_dpkg_control(FILE[, FLAGS[, LINES]])
 
 This is a convenience function to ease using L</parse_dpkg_control>
@@ -600,7 +608,7 @@ Otherwise, this behaves like:
 
  use autodie;
  
- open(my $fd, '<', FILE);
+ open(my $fd, '<:encoding(UTF-8)', FILE); # or '<'
  my @p = parse_dpkg_control($fd, FLAGS, LINES);
  close($fd);
  return @p;
@@ -621,7 +629,39 @@ sub read_dpkg_control {
     return @data;
 }
 
-=item get_deb_control(DEBFILE)
+sub read_dpkg_control_utf8 {
+    my ($file, $flags, $lines) = @_;
+
+    open(my $CONTROL, '<:encoding(UTF-8)', $file);
+    my @data = parse_dpkg_control($CONTROL, $flags, $lines);
+    close($CONTROL);
+
+    return @data;
+}
+
+=item dpkg_deb_has_ctrl_tarfile()
+
+Check if lintian could use dpkg-deb instead of ar and tar
+
+=cut
+
+{
+    my $dpkg_deb_has_ctrl_tarfile_cache;
+
+    sub dpkg_deb_has_ctrl_tarfile {
+        if (not defined($dpkg_deb_has_ctrl_tarfile_cache)) {
+            my $help = safe_qx({'err' => '/dev/null'},'dpkg-deb', '--help');
+            if (index($help, '--ctrl-tarfile') > -1) {
+                $dpkg_deb_has_ctrl_tarfile_cache = 1;
+            } else {
+                $dpkg_deb_has_ctrl_tarfile_cache = 0;
+            }
+        }
+        return $dpkg_deb_has_ctrl_tarfile_cache;
+    }
+}
+
+=item get_deb_info(DEBFILE)
 
 Extracts the control file from DEBFILE and returns it as a hashref.
 
@@ -643,23 +683,35 @@ L</parse_dpkg_control> do.  It can also emit:
 
 =cut
 
-sub get_deb_info {
-    my ($file) = @_;
+{
 
-    # dpkg-deb -f $file is very slow. Instead, we use ar and tar.
-    my $opts = { pipe_out => FileHandle->new };
-    spawn($opts, ['ar', 'p', $file, 'control.tar.gz'],
-        '|', ['tar', '--wildcards', '-xzO', '-f', '-', '*control'])
-      or die "cannot fork to unpack $file: $opts->{exception}\n";
-    my @data = parse_dpkg_control($opts->{pipe_out});
+    sub get_deb_info {
+        my ($file) = @_;
 
-    # Consume all data before exiting so that we don't kill child processes
-    # with SIGPIPE.  This will normally only be an issue with malformed
-    # control files.
-    drain_pipe($opts->{pipe_out});
-    close($opts->{pipe_out});
-    $opts->{harness}->finish();
-    return $data[0];
+        # dpkg-deb -f $file is very slow. Instead, we use ar and tar.
+        my $opts = {
+            fail => 'exception',
+            pipe_out => FileHandle->new
+        };
+
+        if (dpkg_deb_has_ctrl_tarfile()) {
+            spawn(
+                $opts, ['dpkg-deb', '--ctrl-tarfile', $file],
+                '|', ['tar', '--wildcards', '-xO', '-f', '-', '*control']);
+        } else {
+            spawn($opts, ['ar', 'p', $file, 'control.tar.gz'],
+                '|', ['tar', '--wildcards', '-xzO', '-f', '-', '*control']);
+        }
+        my @data = parse_dpkg_control($opts->{pipe_out});
+
+        # Consume all data before exiting so that we don't kill child processes
+        # with SIGPIPE.  This will normally only be an issue with malformed
+        # control files.
+        drain_pipe($opts->{pipe_out});
+        close($opts->{pipe_out});
+        $opts->{harness}->finish();
+        return $data[0];
+    }
 }
 
 =item get_dsc_control (DSCFILE)
@@ -765,19 +817,24 @@ Returns a truth value if STRING can be decoded as valid UTF-8.
 
 =cut
 
-sub is_string_utf8_encoded {
-    my ($str) = @_;
-    if ($str =~ m,\e[-!"\$%()*+./],) {
-        # ISO-2022
-        return 0;
+{
+    my $decoder = Encode::find_encoding('UTF-8');
+    die('No UTF-8 decoder !?') unless ref($decoder);
+
+    sub is_string_utf8_encoded {
+        my ($str) = @_;
+        if ($str =~ m,\e[-!"\$%()*+./],) {
+            # ISO-2022
+            return 0;
+        }
+        eval {$decoder->decode($str, Encode::FB_CROAK);};
+        if ($@) {
+            # fail
+            return 0;
+        }
+        # pass
+        return 1;
     }
-    eval {Encode::decode('UTF-8', $str, Encode::FB_CROAK);};
-    if ($@) {
-        # fail
-        return 0;
-    }
-    # pass
-    return 1;
 }
 
 =item file_is_encoded_in_non_utf8 (...)
@@ -830,8 +887,6 @@ selected few whitelisted variables.
 The list of whitelisted %ENV variables are:
 
  PATH
- INTLTOOL_EXTRACT
- LOCPATH
  LC_ALL (*)
 
 (*) LC_ALL is a special case as clean_env will change its value to
@@ -841,9 +896,9 @@ either "C.UTF-8" or "C" (if CLOC is given and a truth value).
 
 sub clean_env {
     my ($cloc) = @_;
-    my @whitelist = qw(PATH INTLTOOL_EXTRACT LOCPATH);
+    my @whitelist = qw(PATH);
     my %newenv
-      = map { exists $ENV{$_} ? ($_ => $ENV{$_}) : () } (@whitelist, @_);
+      = map { exists $ENV{$_} ? ($_ => $ENV{$_}) : () } (@whitelist);
     %ENV = %newenv;
     $ENV{'LC_ALL'} = 'C.UTF-8';
 
@@ -1037,7 +1092,7 @@ If the tool cannot be found, this sub will cause a trappable error.
 =cut
 
 {
-    my %_CACHE = ();
+    my %_CACHE;
 
     sub locate_helper_tool {
         my ($toolname) = @_;
@@ -1271,7 +1326,7 @@ sub normalize_pkg_path {
         # We are doing CURDIR + LINK_TARGET
         if (substr($dest, 0, 1) eq '/') {
             # Link is absolute
-            # short curcuit $dest eq '/' case.
+            # short circuit $dest eq '/' case.
             return q{} if $dest eq '/';
             $path = $dest;
         } else {
@@ -1412,6 +1467,66 @@ sub unix_locale_split {
     return @parts;
 }
 
+=item pipe_tee(INHANDLE, OUTHANDLES[, OPTS])
+
+Read bytes from INHANDLE and copy them into all of the handles in the
+listref OUTHANDLES. The optional OPTS argument is a hashref of
+options, see below.
+
+The subroutine will continue to read from INHANDLE until it is
+exhausted or an error occurs (either during read or write).  In case
+of errors, a trappable error will be raised.  The handles are left
+open when the subroutine returns, caller must close them afterwards.
+
+Caller should ensure that handles are using "blocking" I/O.  The
+subroutine will use L<sysread|perlfunc/sysread> and
+L<syswrite|perlfunc/syswrite> when reading and writing.
+
+
+OPTS, if given, may contain the following key-value pairs:
+
+=over 4
+
+=item chunk_size
+
+A suggested buffer size for read/write.  If given, it will be to
+sysread as LENGTH argument when reading from INHANDLE.
+
+=back
+
+=cut
+
+sub pipe_tee {
+    my ($in_fd, $out_ref, $opts) = @_;
+    my $read_size = ($opts && $opts->{'chunk_size'}) // 8096;
+    my @outs = @{$out_ref};
+    my $buffer;
+    while (1) {
+        # Disable autodie, because it includes the buffer
+        # exception.  Said buffer will get printed on errors
+        # yielding completely unreadable errors and a terminal
+        # drowned in binary characters.
+        no autodie qw(sysread syswrite);
+        my $rlen = sysread($in_fd, $buffer, $read_size);
+        if (not $rlen) {
+            last if defined($rlen);
+            croak("Failed to read from input handle: $!");
+        }
+        for my $out_fd (@outs) {
+            my $written = 0;
+            while ($written < $rlen) {
+                my $remain = $rlen - $written;
+                my $res = syswrite($out_fd, $buffer, $remain,$written);
+                if (!defined($res)) {
+                    croak("Failed to write to output handle: $!");
+                }
+                $written += $res;
+            }
+        }
+    }
+    return 1;
+}
+
 =item load_state_cache(STATE_DIR)
 
 [Reporting tools only] Load the state cache from STATE_DIR.
@@ -1423,8 +1538,8 @@ sub load_state_cache {
     my $state_file = "$state_dir/state-cache";
     my $state = {};
     my $fd;
-    require YAML::Any;
-    eval {open($fd, '<', $state_file);};
+    require YAML::XS;
+    eval {open($fd, '<:raw', $state_file);};
     if (my $err = $@) {
         if ($err->errno != ENOENT) {
             # Present, but unreadable for some reason
@@ -1433,7 +1548,7 @@ sub load_state_cache {
         # Not present; presume empty
         return $state;
     }
-    eval {$state = YAML::Any::Load(slurp_entire_file($fd, 1));};
+    eval {$state = YAML::XS::Load(slurp_entire_file($fd, 1));};
     # Not sure what Load does in case of issues; perldoc YAML says
     # very little about it.  Based on YAML::Error, I guess it will
     # write stuff to STDERR and use die/croak, but it remains a
@@ -1450,6 +1565,47 @@ sub load_state_cache {
     return $state;
 }
 
+=item save_state_cache(STATE_DIR, STATE)
+
+[Reporting tools only] Save the STATE cache to STATE_DIR.
+
+=cut
+
+sub save_state_cache {
+    my ($state_dir, $state) = @_;
+    my $state_file = "$state_dir/state-cache";
+    my ($tmp_fd, $tmp_path);
+    require YAML::XS;
+
+    ($tmp_fd, $tmp_path) = tempfile('state-cache-XXXXXX', DIR => $state_dir);
+    ## TODO: Should tmp_fd be binmode'd as we use YAML::XS?
+
+    # atomic replacement of the state file; not a substitute for
+    # proper locking, but it will at least ensure that the file
+    # is in a consistent state.
+    eval {
+        print {$tmp_fd} YAML::XS::Dump($state);
+
+        close($tmp_fd) or die("close $tmp_path: $!");
+
+        # There is no secret in this.  Set it to 0644, so it does not
+        # require sudo access on lintian.d.o to read the file.
+        chmod(0644, $tmp_path);
+
+        rename($tmp_path, $state_file)
+          or die("rename $tmp_path -> $state_file: $!");
+    };
+    if (my $err = $@) {
+        if (-e $tmp_path) {
+            # Ignore error as we have a more important one
+            no autodie qw(unlink);
+            unlink($tmp_path);
+        }
+        die($@);
+    }
+    return 1;
+}
+
 =item find_backlog(LINTIAN_VERSION, STATE)
 
 [Reporting tools only] Given the current lintian version and the
@@ -1462,13 +1618,19 @@ processed the package.
 sub find_backlog {
     my ($lintian_version, $state) = @_;
     my (@list, @sorted);
-    for my $group_id (keys(%{$state})) {
+    for my $group_id (keys(%{$state->{'groups'}})) {
         my $last_version = '0';
-        if (exists($state->{$group_id}{'last-processed-by'})) {
-            $last_version = $state->{$group_id}{'last-processed-by'};
+        my $group_data = $state->{'groups'}{$group_id};
+        my $is_out_of_date;
+        if (exists($group_data->{'out-of-date'})) {
+            $is_out_of_date = $group_data->{'out-of-date'};
         }
-        push(@list, [$group_id, $last_version])
+        if (exists($group_data->{'last-processed-by'})) {
+            $last_version = $group_data->{'last-processed-by'};
+        }
+        $is_out_of_date = 1
           if not versions_equal($last_version, $lintian_version);
+        push(@list, [$group_id, $last_version]) if $is_out_of_date;
     }
     @sorted = map { $_->[0] }
       sort { versions_comparator($a->[1], $b->[1]) || $a->[0] cmp $b->[0] }
@@ -1476,6 +1638,17 @@ sub find_backlog {
     return @sorted;
 }
 
+=item untaint(VALUE)
+
+Untaint VALUE
+
+=cut
+
+sub untaint {
+    return $_[0] = $1 if $_[0] =~ m/^(.*)$/;
+    return;
+}
+
 =back
 
 =head1 SEE ALSO
