@@ -53,7 +53,7 @@ use strict;
 use warnings;
 
 my $CLASS = __PACKAGE__;
-my $PROFILE = undef;
+my $PROFILE;
 our @EXPORT = qw(
   load_profile_for_test
 
@@ -92,7 +92,7 @@ my %known_html_tags = map { $_ => 1 } qw(a em i tt);
 # a manual ref.
 my $MANUALS = $Lintian::Tag::Info::MANUALS;
 # lazy-load this (so loading a profile can affect it)
-my %URLS = ();
+my %URLS;
 
 =head1 FUNCTIONS
 
@@ -225,7 +225,7 @@ sub test_check_desc {
             $builder->skip("Special case, $reason");
             $builder->skip("Special case, $reason");
         } elsif ($builder->isnt_eq($ctype, '', "$cname has a type")) {
-            my @bad = ();
+            my @bad;
             # new lines are not allowed, map them to "\\n" for readability.
             $ctype =~ s/\n/\\n/go;
             foreach my $type (split m/\s*+,\s*+/o, $ctype) {
@@ -246,7 +246,7 @@ sub test_check_desc {
                 $builder->is_eq('', $needs,
                     "Translations for $cname ". 'should not have Needs-Info');
             } else {
-                my @bad = ();
+                my @bad;
                 foreach my $need (split m/\s*+,\s*+/o, $needs) {
                     push @bad, $need unless -f "$colldir/$need.desc";
                 }
@@ -264,8 +264,13 @@ sub test_check_desc {
         if ($is_translation) {
             $builder->skip('Skip language specific test');
         } else {
-            $builder->is_eq(check_spelling(undef, $cinfo),
-                0, "$cname Info has no spelling errors");
+            my $mistakes = 0;
+            my $handler = sub {
+                my ($incorrect, $correct) = @_;
+                $builder->diag("Spelling ($cname): $incorrect => $correct");
+                $mistakes++;
+            };
+            $builder->is_eq($mistakes, 0,"$cname Info has no spelling errors");
         }
 
         foreach my $tpara (@tagpara) {
@@ -309,8 +314,15 @@ sub test_check_desc {
                 $builder->skip('Skip language specific test');
                 $builder->skip('Skip language specific test');
             } else {
-                $builder->is_eq(check_spelling(undef, $info),
-                    0, "$content_type $cname: $tag has no spelling errors");
+                my $mistakes = 0;
+                my $handler = sub {
+                    my ($incorrect, $correct) = @_;
+                    $builder->diag(
+                        "Spelling ($cname/$tag): $incorrect => $correct");
+                    $mistakes++;
+                };
+                $builder->is_eq($mistakes, 0,
+                    "$content_type $cname: $tag has no spelling errors");
 
                 $builder->ok($info !~ /(?:^| )(?:[Ww]e|I)\b/,
                     'Tag info does not speak of "I", or "we"')
@@ -620,7 +632,7 @@ sub test_tags_implemented {
     }
 
     foreach my $checkname (@checknames) {
-        my $cs;
+        my ($cs, @tags, $codestr, @missing);
         eval {$cs = Lintian::CheckScript->new($dir, $checkname);};
         if (my $err = $@) {
             $err =~ s/ at .*? line \d+\s*\n//;
@@ -630,9 +642,6 @@ sub test_tags_implemented {
         }
         my $cname = $cs->name;
         my $check = $cs->script_path;
-        my @tags = ();
-        my $codestr;
-        my @missing;
 
         @tags = $cs->tags unless defined $pattern;
         @tags = grep { !m/$pattern/ } $cs->tags
@@ -707,7 +716,7 @@ sub load_profile_for_test {
 
 sub _check_reference {
     my ($refdata) = @_;
-    my @issues = ();
+    my @issues;
 
     unless (%URLS) {
         $MANUALS->known(''); # force it to load the manual refs
@@ -722,9 +731,9 @@ sub _check_reference {
     }
 
     foreach my $reference (split /\s*,\s*/, $refdata) {
-        if (   $reference =~ m,^http://bugs.debian.org/(\d++)$,
+        if (   $reference =~ m,^https?://bugs.debian.org/(\d++)$,
             or $reference
-            =~ m,^http://bugs.debian.org/cgi-bin/bugreport.cgi\?/.*bug=(\d++).*$,
+            =~ m,^https?://bugs.debian.org/cgi-bin/bugreport.cgi\?/.*bug=(\d++).*$,
           ) {
             push @issues, "replace \"$reference\" with \"#$1\"";
         } elsif (exists $URLS{$reference}) {
@@ -764,8 +773,7 @@ sub _find_check {
     }
 
     if (-d $input) {
-        my @result = ();
-        my $regex = undef;
+        my (@result, $regex);
         if ($find_opt->{'want-check-name'}) {
             $regex = qr,^\Q$input\E/*,;
         }
